package ss2011.se.dp.filesystem;

import java.io.File;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

public abstract class AbstractFile
{

    private AbstractFile parent;
    private Set<AbstractFile> children = new HashSet<AbstractFile>();
    private String name;
    private Printer printer = new DepthFirstPrinter();

    public AbstractFile(AbstractFile parent, String name)
    {
        this.parent = parent;
        this.name = name;
    }

    public Set<AbstractFile> getChildren()
    {
        return Collections.unmodifiableSet(children);
    }

    public AbstractFile getParent()
    {
        return parent;
    }

    public abstract long getSize();

    public boolean addChild(AbstractFile file)
    {
        return children.add(file);
    }

    public String getName()
    {
        return name;
    }

    public AbstractFile createFile(AbstractFile parent, File file)
    {
        if (!file.isDirectory())
        {
            if (file.getName().toLowerCase().endsWith(".jar"))
            {
                return new JARArchive(parent, file);
            }
            if (file.getName().toLowerCase().endsWith(".zip"))
            {
                return new ZIPArchive(parent, file);
            } else
            {
                return new RegularFile(parent, file.getName(), file.length());
            }
        } else
        {
            return new Directory(parent, file);
        }
    }

    public void printTree()
    {
        if (printer != null)
        {
            printer.printTree(this);
        }
    }

    public void printTree(int maxdepth)
    {
        if (printer != null)
        {
            printer.printTree(this, maxdepth);
        }
    }

    public Printer getPrinter()
    {
        return printer;
    }

    public void setPrinter(Printer printer)
    {
        this.printer = printer;
    }
}
