package ss2011.se.dp.ha4;

import junit.framework.TestCase;
import org.junit.Test;
import ss2011.se.dp.filesystem.Directory;
import ss2011.se.dp.ha4.a1.Ampel;
import ss2011.se.dp.ha4.a2.Bus;
import ss2011.se.dp.ha4.a2.Message;
import ss2011.se.dp.ha4.a2.Subscriber;

import java.io.File;

public class Hausaufgabe4Test extends TestCase
{

    /**
     * Teilaufgabe 1:
     * <p/>
     * Die Klasse ist so umzubauen, dass keine Fallunterscheidungen mehr enthalten sind, sondern das
     * Strategy Pattern verwendet wird. Das Verhalten ist also in ein austauschbares Objekt auszulagern,
     * z.B. in eine Klasse "AmpelFarbe" (und Delegation der beiden Methoden). Das kann in diesem Fall
     * auch durch das Sprachkonstrukt 'enum' realisiert werden. Es duerfen keine
     * public static final int ...-Felder
     * mehr in der Klasse Ampel vorhanden sein!
     * <p/>
     * Falls noetig, koennen die assert-Methoden dementsprechend hier angepasst werden. Ansonsten
     * soll der Test unverndert bleiben.
     */
    @Test
    public void testAmpelWithStrategyPattern()
    {
        Ampel ampel = new Ampel();
        System.out.println("Ampel zeigt: " + ampel.getFarbeAsString());
        assertEquals(Ampel.ROT, ampel.getFarbe());

        ampel.weiterschalten();
        System.out.println("Ampel zeigt: " + ampel.getFarbeAsString());
        assertEquals(Ampel.GELB, ampel.getFarbe());

        ampel.weiterschalten();
        System.out.println("Ampel zeigt: " + ampel.getFarbeAsString());
        assertEquals(Ampel.GRUEN, ampel.getFarbe());

        // Einfache Variante ist genug: Nach Gruen folgt direkt Rot
        ampel.weiterschalten();
        System.out.println("Ampel zeigt: " + ampel.getFarbeAsString());
        assertEquals(Ampel.ROT, ampel.getFarbe());
    }

    /**
     * Teilaufgabe 2:
     *
     * Gegeben ist ein Bus System: Man kann ueber den Bus Nachrichten an alle oder bestimmte Teilnehmer
     * senden. Jeder Teilnehmer hat eine (eindeutige) Adresse (int) und soll die letzte empfangene
     * Nachricht abspeichern. Nachrichten werden beim Zustellen kopiert!
     *
     * Es gibt zwei Moeglichkeiten die Datenstruktur zu implementieren:
     * 1) 1:1 Assoziation zwischen Bus und dem ersten Subscriber, und eine next-Assozation zu allen
     *    weiteren Subscribern
     * 2) 1:n Assoziation zwischen Bus und Subscribern
     *
     * Waehle eine von den beiden Moeglichkeiten, gib im Methodenrumpf(!) von Bus.add(Subscriber) an
     * WARUM diese Moeglichkeit gewaehlt wurde (z.B. Vorteile dieser und Nachteile der anderen Loesung
     * aufzaehlen).
     *
     * Dann:
     * - implementiere Bus.add passend zur gewaehlten Datenstruktur
     * - implementiere Bus.broadcast(Message). Beim Zustellen der Nachrichten an einen Teilnehmer soll
     * die Nachrichten-Instanz kopiert werden!
     *
     * - implementiere Bus.send(int,Message)
     * unter Beruecksichtigung des ChainOfResponsibility-Patterns; d.h. send() soll eine Methode auf
     * dem Subscriber aufrufen, der dann selber entscheidet ob die Message fuer ihn bestimmt ist, und
     * im diesem Falle die weitere Zustellung unterbricht.
     *
     * Hinweis: Implementiere notwendigerweise Message.equals(), um die vorgegebenen Assert-Methoden
     * halten zu koennen.
     */
    @Test
    public void testBusWithHookAndChainOfResponsibility()
    {
        Bus bus = new Bus();
        Subscriber s0 = new Subscriber(0);
        bus.add(s0);
        Subscriber s1 = new Subscriber(1);
        bus.add(s1);
        Subscriber s2 = new Subscriber(2);
        bus.add(s2);

        Message m = new Message("Hallo!?");

        assertNull(s0.getLastReceivedMessage());
        assertNull(s1.getLastReceivedMessage());
        assertNull(s2.getLastReceivedMessage());

        bus.broadcast(m);

        assertNotNull(s0.getLastReceivedMessage());
        assertNotNull(s1.getLastReceivedMessage());
        assertNotNull(s2.getLastReceivedMessage());
        assertEquals(m, s0.getLastReceivedMessage());
        assertEquals(m, s1.getLastReceivedMessage());
        assertEquals(m, s2.getLastReceivedMessage());
        assertNotSame(m, s0.getLastReceivedMessage());

        Message m2 = new Message("Nur fr Dich!");
        bus.send(1, m2);
        assertEquals(m, s0.getLastReceivedMessage());
        assertEquals(m2, s1.getLastReceivedMessage());
        assertEquals(m, s2.getLastReceivedMessage());
    }

    /**
     * Zusatzaufgabe zum Strategy Pattern:
     * <p/>
     * Implementiere eine Breitensuche als zweite Strategie fuer die Ausgabe.
     * Es sollen zuerst alle direkt enthaltenen
     * Dateien und Verzeichnisse ausgegeben werden, dann die naechste Ebene usw...
     * Dafuer ist eine neue Printer-Klasse (z.B. BreadthFirstPrinter)
     * mit derselben Printer-Schnittstelle zu erstellen und zu implementieren.
     * <p/>
     * Weiterhin soll die Ausgabetiefe per Parameter begrenzt werden. Beide Verfahren
     * sollen dies unterstuetzen. Passe beide Printer dementsprechend an.
     */
    @Test
    public void testPrintTreeAsStrategy()
    {
        Directory d = new Directory(null, new File("."));
        d.printTree(); // gibt die Dateien und Verzeichnisse nach Tiefensuche-Reihenfolge aus.

        d.printTree(2);

        // d.setPrinter(new BreathFirstPrinter());
        d.printTree(); // soll die Dateien und Verzeichnisse nach Breitensuche-Reihenfolge ausgeben

        d.printTree(2);
    }

}
